require_dependency "issues_controller"

module IssuesControllerPatch
  def self.included(base)
    base.class_eval do

      IssuesController.filter_chain.each do |filter|
        if filter.method == :find_project
          filter.options[:only] << "import"
          filter.options[:only] << "import_template"
        end
      end

      def import
        flash_errors = []
        @file = params[:file].path unless params[:file].nil?
        if request.post?
          file_content = @file && File.open(@file).read
          if file_content.nil?
            flash_errors << l(:error_nothing_to_import)
          else
            @issues = []
            issue = nil
            line_number = 1
            file_content.split(/[\n\r]/).each do |line|
              line.strip!
              line_downcase = line.downcase
              if line_downcase == "issue:"
                unless issue.nil?
                  @issues << issue
                  issue = nil
                end
                issue = Issue.new({:project_id => @project.id, :author_id => User.current.id})
              elsif line_downcase.starts_with?("tracker:")
                tracker = Tracker.find_by_name(line["tracker:".length, line.length].strip!)
                if issue.nil? or tracker.nil?
                  flash_errors << l(:error_importing, :line_number => line_number, :line => line)
                else
                  issue.tracker = tracker
                end
              elsif line_downcase.starts_with?("subject:")
                subject = line["subject:".length, line.length].strip!
                if issue.nil? or subject.empty?
                  flash_errors << l(:error_importing, :line_number => line_number, :line => line)
                else
                  issue.subject = subject
                end
              elsif line_downcase.starts_with?("description:")
                description = line["description:".length, line.length].gsub!("\\n", "\n").strip!
                if issue.nil?
                  flash_errors << l(:error_importing, :line_number => line_number, :line => line)
                else
                  issue.description = description
                end
              elsif line_downcase.starts_with?("due date:")
                due_date = line["due date:".length, line.length].strip!
                if issue.nil? or ((issue.due_date = due_date).nil? and !(due_date.nil?))
                  flash_errors << l(:error_importing, :line_number => line_number, :line => line)
                end
              elsif line_downcase.starts_with?("category:")
                category = IssueCategory.find_by_name_and_project_id(line["category:".length, line.length].strip!, @project.id)
                if issue.nil? or category.nil?
                  flash_errors << l(:error_importing, :line_number => line_number, :line => line)
                else
                  issue.category = category
                end
              elsif line_downcase.starts_with?("status:")
                status = IssueStatus.find_by_name(line["status:".length, line.length].strip!)
                if issue.nil? or status.nil?
                  flash_errors << l(:error_importing, :line_number => line_number, :line => line)
                else
                  issue.status = status
                end
              elsif line_downcase.starts_with?("assigned to:")
                assigned_to = User.find_by_login(line["assigned to:".length, line.length].strip!)
                unless assigned_to.nil?
                  assignable = false
                  assigned_to.roles_for_project(@project).each do |role|
                    assignable = true if role.assignable
                  end
                end
                if issue.nil? or !assignable or !(assigned_to.member_of?(@project))
                  flash_errors << l(:error_importing, :line_number => line_number, :line => line)
                else
                  issue.assigned_to = assigned_to
                end
              elsif line_downcase.starts_with?("priority:")
                priority = Enumeration.find_by_name_and_type_and_active(line["priority:".length, line.length].strip!, "IssuePriority", 1)
                if issue.nil? or priority.nil?
                  flash_errors << l(:error_importing, :line_number => line_number, :line => line)
                else
                  issue.priority = priority
                end
              elsif line_downcase.starts_with?("fixed version:")
                fixed_version = Version.find_by_name_and_project_id(line["fixed version:".length, line.length].strip!, @project.id)
                if issue.nil? or fixed_version.nil?
                  flash_errors << l(:error_importing, :line_number => line_number, :line => line)
                else
                  issue.fixed_version = fixed_version
                end
              elsif line_downcase.starts_with?("author:")
                author = User.find_by_login(line["author:".length, line.length].strip!)
                author = nil unless author.nil? or author.allowed_to?({:controller => :issues, :action => :new}, @project)
                if issue.nil? or author.nil?
                  flash_errors << l(:error_importing, :line_number => line_number, :line => line)
                else
                  issue.author = author
                end
              elsif line_downcase.starts_with?("created on:")
                created_on = line["created on:".length, line.length].strip!
                if issue.nil? or ((issue.created_on = created_on).nil? and !(created_on.nil?))
                  flash_errors << l(:error_importing, :line_number => line_number, :line => line)
                end
              elsif line_downcase.starts_with?("updated on:")
                updated_on = line["updated on:".length, line.length].strip!
                if issue.nil? or ((issue.updated_on = updated_on).nil? and !(updated_on.nil?))
                  flash_errors << l(:error_importing, :line_number => line_number, :line => line)
                end
              elsif line_downcase.starts_with?("start date:")
                start_date = line["start date:".length, line.length].strip!
                if issue.nil? or ((issue.start_date = start_date).nil? and !(start_date.nil?))
                  flash_errors << l(:error_importing, :line_number => line_number, :line => line)
                end
              elsif line_downcase.starts_with?("done ratio:")
                done_ratio = line["done ratio:".length, line.length].strip!
                if issue.nil? or ((issue.done_ratio = done_ratio).nil? and !(done_ratio.nil?)) or (issue.done_ratio < 0 or issue.done_ratio > 100)
                  flash_errors << l(:error_importing, :line_number => line_number, :line => line)
                end
              elsif line_downcase.starts_with?("estimated hours:")
                estimated_hours = line["estimated hours:".length, line.length].strip!
                if issue.nil? or ((issue.estimated_hours = estimated_hours).nil? and !(estimated_hours.nil?)) or (issue.estimated_hours < 0.0)
                  flash_errors << l(:error_importing, :line_number => line_number, :line => line)
                end
              end
              line_number += 1
            end
            unless issue.nil?
              @issues << issue
            end
            if @issues.length > 0 and flash_errors.length == 0
              goOn = true
              @issues.each do |issue|
                issue.validate
                unless issue.errors.empty?
                  goOn = false
                  flash_errors << l(:error_validating, :issue => issue.subject, :error => issue.errors.full_messages.join(", "))
                end
              end
              if goOn
                @issues.each do |issue|
                  issue.save!
                end
                flash[:notice] = l(:notice_import_ok)
              end
            end
          end
        end
        flash[:error] = flash_errors.join("<br />\n") unless flash_errors.length == 0
      end

      def import_template
        render(:layout => false)
      end

    end
  end
end
